#include "wx/wx.h"
#include <wx/glcanvas.h>
#include <GL/glu.h>


class MyApp : public wxApp
{
public:
    virtual bool OnInit();
};

class TestGLCanvas;


class MyFrame : public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
            const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);

    TestGLCanvas* m_canvas;

    DECLARE_EVENT_TABLE()
};


class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:
 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas");
   
 ~TestGLCanvas(void);

 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 
 void Render( void );
 void InitGL(void);
   
private:
  bool   m_init;


DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(TestGLCanvas, wxGLCanvas)
  EVT_SIZE(TestGLCanvas::OnSize)
  EVT_PAINT(TestGLCanvas::OnPaint)
  EVT_ERASE_BACKGROUND(TestGLCanvas::OnEraseBackground)
  EVT_ENTER_WINDOW( TestGLCanvas::OnEnterWindow )
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
END_EVENT_TABLE()



IMPLEMENT_APP(MyApp)


static void Reshape(int width, int height)
{

  GLfloat mprojecao[16];
  glViewport(0,0,width,height); 
  glMatrixMode( GL_PROJECTION ); 
  glLoadIdentity(); 

  mprojecao [0]=1;
  mprojecao [1]= 0;
  mprojecao [2]= 0;
  mprojecao [3]= 0;

  mprojecao [4]= 0;
  mprojecao [5]= 1;
  mprojecao [6]= 0;
  mprojecao [7]= 0;

  mprojecao [8]= 1.4344/4;
  mprojecao [9]= 1.4344/4;
  mprojecao [10]= 0;
  mprojecao [11]= 0;
  
  mprojecao [12]= 0;
  mprojecao [13]= 0;
  mprojecao [14]= 0;
  mprojecao [15]= 1;

  

  glLoadMatrixf (mprojecao);
  glClearColor(0.0f,0.0f,0.0f,0.0f); 
  glClear( GL_COLOR_BUFFER_BIT  );

//
  glClear(GL_DEPTH_BUFFER_BIT); 
  glEnable(GL_DEPTH_TEST);
  
  glMatrixMode( GL_MODELVIEW ); 
  glLoadIdentity();

}



bool MyApp::OnInit()
{

  MyFrame *frame = new MyFrame(NULL, "Exemplo Basico", wxPoint(50, 50),
                               wxSize(350, 350));

  wxMenu *winMenu = new wxMenu;

  winMenu->Append(wxID_EXIT, "&Fechar");

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(winMenu, "&Janela");

  frame->SetMenuBar(menuBar);

  frame->m_canvas = new TestGLCanvas(frame, -1, wxPoint(0, 0), wxSize(500, 500));

  frame->Show(TRUE);
  
  return TRUE;
}


MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                 const wxSize& size, long style) 
         : wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = NULL;
}

TestGLCanvas::TestGLCanvas(wxWindow *parent, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name):
  wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}


TestGLCanvas::~TestGLCanvas()
{
}

void MyFrame::OnExit(wxCommandEvent& event)
{
    Destroy();
}

void TestGLCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

void TestGLCanvas::OnEraseBackground(wxEraseEvent& event)
{
}

void TestGLCanvas::OnPaint( wxPaintEvent& event )
{
    Render();
}

void TestGLCanvas::Render()
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();

    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }
    


  glColor3f(0.2, 0.3, 0.7);

  glBegin(GL_POLYGON);
     glVertex3f(0.25,0.25,0.25);
     glVertex3f(0.25,0.25,0.0);
     glVertex3f(0.0,0.25,0.0);
     glVertex3f(0.0,0.25,0.25);
  glEnd();                        

  glColor3f(0.4, 0.2, 0.8);

  glBegin(GL_POLYGON);
     glVertex3f(0.25,0.25,0.25);
     glVertex3f(0.25,0.0,0.25);
     glVertex3f(0.25,0.0,0.0);
     glVertex3f(0.25,0.25,0.0);
  glEnd();                        
                        
  glColor3f(0.2, 0.3, 0.4);

  glBegin(GL_POLYGON);
     glVertex3f(0.25,0.25,0.25);
     glVertex3f(0.25,0.0,0.25);
     glVertex3f(0.0,0.0,0.25);
     glVertex3f(0.0,0.25,0.25);
  glEnd();                        

  glColor3f(0.2, 0.3, 0.55);

  glBegin(GL_POLYGON);
     glVertex3f(0.0,0.0,0.0);
     glVertex3f(0.0,0.25,0.0);
     glVertex3f(0.25,0.25,0.0);
     glVertex3f(0.25,0.0,0.0);
  glEnd();                        

  glColor3f(0.2, 0.45, 0.4);

  glBegin(GL_POLYGON);
     glVertex3f(0.0,0.0,0.0);
     glVertex3f(0.25,0.0,0.0);
     glVertex3f(0.25,0.0,0.25);
     glVertex3f(0.0,0.0,0.25);
  glEnd();                        

  glColor3f(0.33, 0.3, 0.4);

  glBegin(GL_POLYGON);
     glVertex3f(0.0,0.0,0.0);
     glVertex3f(0.0,0.0,0.25);
     glVertex3f(0.0,0.25,0.25);
     glVertex3f(0.0,0.25,0.0);
  glEnd();  
  
  glColor3f(0.0, 1.0, 0.0);

  glBegin(GL_POLYGON);
     glVertex3f(0.25,0.25,0.25);
     glVertex3f(0.25,0.0,0.25);
     glVertex3f(0.25,0.0,0.0);
     glVertex3f(0.25,0.25,0.0);
  glEnd();                        
                        
  glColor3f(1.0, 0.0, 0.0);

  glBegin(GL_POLYGON);
     glVertex3f(0.25,0.25,0.25);
     glVertex3f(0.25,0.0,0.25);
     glVertex3f(0.0,0.0,0.25);
     glVertex3f(0.0,0.25,0.25);
  glEnd();                        

  glFlush();
  SwapBuffers();
}

void TestGLCanvas::InitGL()
{
   SetCurrent();
   glClearColor(0.0, 0.0, 0.0, 0.0);
   

}

void TestGLCanvas::OnSize(wxSizeEvent& event)
{
    int width, height;
    GetClientSize(& width, & height);
    Reshape(width, height);
        if (height==0)  
	{ 
                height=1;   
	}  


#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
    }
}


